@@ -6,6 +6,7 @@
 
 #pragma once
 
+#include <QObject>
 #include <QPointer>
 #include <QQuickItem>
 #include <QVariant>
@@ -55,6 +56,37 @@ class ColumnViewAttached : public QObject
      */
     Q_PROPERTY(qreal reservedSpace READ reservedSpace WRITE setReservedSpace NOTIFY reservedSpaceChanged FINAL)
 
+    /*!
+     * \qmlattachedproperty real ColumnView::minimumWidth
+     *
+     * When a column is interactiveResizeEnabled, when resizing manually it will bound
+     * the column size between minimumWidth and maximumWidth
+     *
+     * \since 6.16
+     */
+    Q_PROPERTY(qreal minimumWidth READ minimumWidth WRITE setMinimumWidth NOTIFY minimumWidthChanged FINAL)
+
+    /*!
+     * \qmlattachedproperty real ColumnView::preferredWidth
+     *
+     * When we are in DynamicColumns mode, the columns will be resized to preferredWidth.
+     * When the column is resized with the mouse, this property will be updated by the drag
+     * And will always be between minimumWidth and maximumWidth
+     *
+     * \since 6.16
+     */
+    Q_PROPERTY(qreal preferredWidth READ preferredWidth WRITE setPreferredWidth NOTIFY preferredWidthChanged FINAL)
+
+    /*!
+     * \qmlattachedproperty real ColumnView::maximumWidth
+     *
+     * When a column is interactiveResizeEnabled, when resizing manually it will bound
+     * the column size between minimumWidth and maximumWidth
+     *
+     * \since 6.16
+     */
+    Q_PROPERTY(qreal maximumWidth READ maximumWidth WRITE setMaximumWidth NOTIFY maximumWidthChanged FINAL)
+
     /*!
      * \qmlattachedproperty bool ColumnView::preventStealing
      *
@@ -89,6 +121,40 @@ class ColumnViewAttached : public QObject
      */
     Q_PROPERTY(bool inViewport READ inViewport NOTIFY inViewportChanged FINAL)
 
+    /*!
+     * \qmlattachedproperty bool ColumnView::interactiveResizeEnabled
+     *
+     * True if this column supports interactive resize with mouse.
+     *
+     * If the sizes should be saved and restored on application startup,
+     * it is the application responsibility to implement this, for instance exposing
+     * and object with properties for the desired column widths which
+     * reads and writes to the config. at the main Component.onCompleted of the
+     * ApplicationWindow root Item it will apply the preferred sizes of the desired pages.
+     *
+     * The property will be written at interactiveResizingChanged signal handler.
+     *
+     * \since 6.16
+     */
+    Q_PROPERTY(bool interactiveResizeEnabled READ interactiveResizeEnabled WRITE setInteractiveResizeEnabled NOTIFY interactiveResizeEnabledChanged)
+
+    /*!
+     * \qmlattachedproperty bool ColumnView::interactiveResizing
+     *
+     * True when the user is actively resizing a column with the mouse.
+     * If the application wants to write the column sizes to config,
+     * it can sync to some global object property at interactiveResizingChanged:
+     * \qml
+     *      Kirigami.ColumnView.onInteractiveResizingChanged: {
+     *          if (!Kirigami.ColumnView.interactiveResizing) {
+     *              myAppObject.leftColumnWidth = page.implicitWidth;
+     *          }
+     *      }
+     * \endqml
+     * \since 6.16
+     */
+    Q_PROPERTY(bool interactiveResizing READ interactiveResizing WRITE setInteractiveResizing NOTIFY interactiveResizingChanged)
+
     /*!
      * \qmlattachedproperty Item ColumnView::globalHeader
      */
@@ -112,6 +178,15 @@ public:
     qreal reservedSpace() const;
     void setReservedSpace(qreal space);
 
+    qreal minimumWidth() const;
+    void setMinimumWidth(qreal space);
+
+    qreal preferredWidth() const;
+    void setPreferredWidth(qreal space);
+
+    qreal maximumWidth() const;
+    void setMaximumWidth(qreal space);
+
     ColumnView *view();
     void setView(ColumnView *view);
 
@@ -131,6 +206,12 @@ public:
     bool inViewport() const;
     void setInViewport(bool inViewport);
 
+    bool interactiveResizeEnabled() const;
+    void setInteractiveResizeEnabled(bool interactive);
+
+    bool interactiveResizing() const;
+    void setInteractiveResizing(bool interactive);
+
     QQuickItem *globalHeader() const;
     void setGlobalHeader(QQuickItem *header);
 
@@ -141,11 +222,16 @@ Q_SIGNALS:
     void indexChanged();
     void fillWidthChanged();
     void reservedSpaceChanged();
+    void minimumWidthChanged();
+    void preferredWidthChanged();
+    void maximumWidthChanged();
     void viewChanged();
     void preventStealingChanged();
     void pinnedChanged();
     void scrollIntention(ScrollIntentionEvent *event);
     void inViewportChanged();
+    void interactiveResizeEnabledChanged();
+    void interactiveResizingChanged();
     void globalHeaderChanged(QQuickItem *oldHeader, QQuickItem *newHeader);
     void globalFooterChanged(QQuickItem *oldFooter, QQuickItem *newFooter);
 
@@ -153,6 +239,9 @@ private:
     int m_index = -1;
     bool m_fillWidth = false;
     qreal m_reservedSpace = 0;
+    qreal m_minimumWidth = -1;
+    qreal m_preferredWidth = -1;
+    qreal m_maximumWidth = -1;
     QPointer<ColumnView> m_view;
     QPointer<QQuickItem> m_originalParent;
     bool m_customFillWidth = false;
@@ -161,6 +250,8 @@ private:
     bool m_preventStealing = false;
     bool m_pinned = false;
     bool m_inViewport = false;
+    bool m_interactiveResizeEnabled = false;
+    bool m_interactiveResizing = false;
     QPointer<QQuickItem> m_globalHeader;
     QPointer<QQuickItem> m_globalFooter;
 };
@@ -348,6 +439,26 @@ class ColumnView : public QQuickItem
      * every item declared inside the view, both visual and non-visual items
      */
     Q_PROPERTY(QQmlListProperty<QObject> contentData READ contentData FINAL)
+
+    /*!
+     * \qmlproperty string ColumnView::savedState
+     *
+     * A JSon serialization of the columns widths.
+     * When columnResizeMode is DynamicColumns, columns can be resized via mouse.
+     * When this is the case, the sizes should be saved and restored on application startup.
+     *
+     * Example code for saving and restore, assuming that the application exposes a singleton
+     * called App that has a "columnViewState" property that reads and writes to the config file:
+     *
+     * \qml
+     *      Component.onCompleted: pageStack.columnView.savedState = App.columnViewState;
+     *      columnView.onSavedStateChanged: {
+     *          App.columnViewState = pageStack.columnView.savedState;
+     *      }
+     * \endqml
+     */
+    Q_PROPERTY(QString savedState READ savedState WRITE setSavedState NOTIFY savedStateChanged)
+
     Q_CLASSINFO("DefaultProperty", "contentData")
 
 public:
@@ -414,6 +525,19 @@ public:
     bool acceptsMouse() const;
     void setAcceptsMouse(bool accepts);
 
+    /*!
+     * \qmlmethod Item ColumnView::get(int index)
+     *
+     * \brief This method gets the item at index, if available
+     *
+     * Note that if an invalid index is passed (either negative or greater
+     * then depth - 1) nullptr will be returned
+     *
+     * \a index The index of the item to get.
+     * \since 6.16
+     */
+    Q_INVOKABLE QQuickItem *get(int index);
+
     /*!
      * \qmlmethod Item ColumnView::pop(var item)
      *
@@ -493,6 +617,9 @@ public:
      */
     QQuickItem *removeItem(int index);
 
+    QString savedState();
+    void setSavedState(const QString &state);
+
     // QML attached property
     static ColumnViewAttached *qmlAttachedProperties(QObject *object);
 
@@ -513,7 +640,7 @@ public Q_SLOTS:
      * to remove an item by its index. Otherwise the argument should be the item
      * itself to be removed itself, and this method will dispatch to removeItem(QQuickItem *item).
      *
-     * \a index the index of the item which should be removed, or the item itself
+     * \a item the item which should be removed
      *
      * Returns the removed item
      */
@@ -620,6 +747,7 @@ Q_SIGNALS:
     void trailingVisibleItemChanged();
     void topPaddingChanged();
     void bottomPaddingChanged();
+    void savedStateChanged();
 
 private:
     static void contentChildren_append(QQmlListProperty<QQuickItem> *prop, QQuickItem *object);
@@ -636,6 +764,7 @@ private:
 
     ContentItem *m_contentItem;
     QPointer<QQuickItem> m_currentItem;
+    QHash<int, qreal> m_state;
 
     int m_currentIndex = -1;
     qreal m_topPadding = 0;
