@@ -21,6 +21,19 @@ PagePool::PagePool(QObject *parent)
 
 PagePool::~PagePool()
 {
+    for (QQmlComponent *component : std::as_const(m_componentForUrl)) {
+        component->deleteLater();
+    }
+    m_componentForUrl.clear();
+
+    for (QQuickItem *item : std::as_const(m_itemForUrl)) {
+        if (!item->parentItem()) {
+            item->deleteLater();
+        } else {
+            QQmlEngine::setObjectOwnership(item, QQmlEngine::JavaScriptOwnership);
+        }
+    }
+    m_itemForUrl.clear();
 }
 
 QUrl PagePool::lastLoadedUrl() const
@@ -33,10 +46,9 @@ QQuickItem *PagePool::lastLoadedItem() const
     return m_lastLoadedItem;
 }
 
-QList<QObject *> PagePool::items() const
+QList<QQuickItem *> PagePool::items() const
 {
-    auto items = m_itemForUrl.values();
-    return QList<QObject *>(items.cbegin(), items.cend());
+    return m_itemForUrl.values();
 }
 
 QList<QUrl> PagePool::urls() const
@@ -68,109 +80,9 @@ QQuickItem *PagePool::loadPage(const QString &url, QJSValue callback)
     return loadPageWithProperties(url, QVariantMap(), callback);
 }
 
-QQuickItem *PagePool::loadPageWithProperties(const QString &url, const QVariantMap &properties, QJSValue callback)
+QQuickItem *PagePool::allocatePage(QQmlComponent *component, const QVariantMap &properties)
 {
-    Q_ASSERT(qmlEngine(this));
-    QQmlContext *ctx = QQmlEngine::contextForObject(this);
-    Q_ASSERT(ctx);
-
-    const QUrl actualUrl = resolvedUrl(url);
-
-    auto found = m_itemForUrl.find(actualUrl);
-    if (found != m_itemForUrl.end()) {
-        m_lastLoadedUrl = found.key();
-        m_lastLoadedItem = found.value();
-
-        if (callback.isCallable()) {
-            QJSValueList args = {qmlEngine(this)->newQObject(found.value())};
-            callback.call(args);
-            Q_EMIT lastLoadedUrlChanged();
-            Q_EMIT lastLoadedItemChanged();
-            // We could return the item, but for api coherence return null
-            return nullptr;
-
-        } else {
-            Q_EMIT lastLoadedUrlChanged();
-            Q_EMIT lastLoadedItemChanged();
-            return found.value();
-        }
-    }
-
-    QQmlComponent *component = m_componentForUrl.value(actualUrl);
-
-    if (!component) {
-        component = new QQmlComponent(qmlEngine(this), actualUrl, QQmlComponent::PreferSynchronous);
-    }
-
-    if (component->status() == QQmlComponent::Loading) {
-        if (!callback.isCallable()) {
-            component->deleteLater();
-            m_componentForUrl.remove(actualUrl);
-            return nullptr;
-        }
-
-        connect(component, &QQmlComponent::statusChanged, this, [this, component, callback, properties](QQmlComponent::Status status) mutable {
-            if (status != QQmlComponent::Ready) {
-                qCWarning(KirigamiLog) << component->errors();
-                m_componentForUrl.remove(component->url());
-                component->deleteLater();
-                return;
-            }
-            QQuickItem *item = createFromComponent(component, properties);
-            if (item) {
-                QJSValueList args = {qmlEngine(this)->newQObject(item)};
-                callback.call(args);
-            }
-
-            if (m_cachePages) {
-                component->deleteLater();
-            } else {
-                m_componentForUrl[component->url()] = component;
-            }
-        });
-
-        return nullptr;
-
-    } else if (component->status() != QQmlComponent::Ready) {
-        qCWarning(KirigamiLog) << component->errors();
-        return nullptr;
-    }
-
-    QQuickItem *item = createFromComponent(component, properties);
-    if (!item) {
-        return nullptr;
-    }
-
-    if (m_cachePages) {
-        component->deleteLater();
-        QQmlEngine::setObjectOwnership(item, QQmlEngine::CppOwnership);
-        m_itemForUrl[component->url()] = item;
-        m_urlForItem[item] = component->url();
-        Q_EMIT itemsChanged();
-        Q_EMIT urlsChanged();
-
-    } else {
-        m_componentForUrl[component->url()] = component;
-        QQmlEngine::setObjectOwnership(item, QQmlEngine::JavaScriptOwnership);
-    }
-
-    m_lastLoadedUrl = actualUrl;
-    m_lastLoadedItem = item;
-    Q_EMIT lastLoadedUrlChanged();
-    Q_EMIT lastLoadedItemChanged();
-
-    if (callback.isCallable()) {
-        QJSValueList args = {qmlEngine(this)->newQObject(item)};
-        callback.call(args);
-        // We could return the item, but for api coherence return null
-        return nullptr;
-    }
-    return item;
-}
-
-QQuickItem *PagePool::createFromComponent(QQmlComponent *component, const QVariantMap &properties)
-{
-    QQmlContext *ctx = QQmlEngine::contextForObject(this);
+    const auto ctx = qmlContext(this);
     Q_ASSERT(ctx);
 
     QObject *obj = component->createWithInitialProperties(properties, ctx);
@@ -190,13 +102,104 @@ QQuickItem *PagePool::createFromComponent(QQmlComponent *component, const QVaria
         return nullptr;
     }
 
+    if (m_cachePages) {
+        QQmlEngine::setObjectOwnership(item, QQmlEngine::CppOwnership);
+        m_itemForUrl[component->url()] = item;
+        m_urlForItem[item] = component->url();
+        Q_EMIT itemsChanged();
+        Q_EMIT urlsChanged();
+
+    } else {
+        QQmlEngine::setObjectOwnership(item, QQmlEngine::JavaScriptOwnership);
+    }
+
+    m_lastLoadedUrl = component->url();
+    m_lastLoadedItem = item;
+    Q_EMIT lastLoadedUrlChanged();
+    Q_EMIT lastLoadedItemChanged();
+
+    return item;
+}
+
+QQuickItem *PagePool::loadPageWithProperties(const QString &url, const QVariantMap &properties, QJSValue callback)
+{
+    const auto engine = qmlEngine(this);
+    Q_ASSERT(engine);
+
+    const QUrl actualUrl = resolvedUrl(url);
+
+    auto found = m_itemForUrl.find(actualUrl);
+    if (found != m_itemForUrl.end()) {
+        m_lastLoadedUrl = found.key();
+        m_lastLoadedItem = found.value();
+
+        if (callback.isCallable()) {
+            QJSValueList args = {engine->newQObject(found.value())};
+            callback.call(args);
+            Q_EMIT lastLoadedUrlChanged();
+            Q_EMIT lastLoadedItemChanged();
+            // We could return the item, but for api coherence return null
+            return nullptr;
+
+        } else {
+            Q_EMIT lastLoadedUrlChanged();
+            Q_EMIT lastLoadedItemChanged();
+            return found.value();
+        }
+    }
+
+    // Note component->url() and actualUrl will be the same.
+    QQmlComponent *&component = m_componentForUrl[actualUrl];
+    if (!component) {
+        component = new QQmlComponent(engine, actualUrl, QQmlComponent::PreferSynchronous);
+    }
+
+    if (component->status() == QQmlComponent::Loading) {
+        if (!callback.isCallable()) {
+            component->deleteLater();
+            m_componentForUrl.remove(component->url());
+            return nullptr;
+        }
+
+        connect(component, &QQmlComponent::statusChanged, this, [this, engine, component, callback, properties](QQmlComponent::Status status) mutable {
+            if (status != QQmlComponent::Ready) {
+                qCWarning(KirigamiLog) << component->errors();
+                m_componentForUrl.remove(component->url());
+                component->deleteLater();
+                return;
+            }
+
+            QQuickItem *item = allocatePage(component, properties);
+            if (item) {
+                QJSValueList args = {engine->newQObject(item)};
+                callback.call(args);
+            }
+        });
+
+        return nullptr;
+
+    } else if (component->status() != QQmlComponent::Ready) {
+        qCWarning(KirigamiLog) << component->errors();
+        return nullptr;
+    }
+
+    QQuickItem *item = allocatePage(component, properties);
+    if (!item) {
+        return nullptr;
+    }
+
+    if (callback.isCallable()) {
+        QJSValueList args = {engine->newQObject(item)};
+        callback.call(args);
+        // We could return the item, but for api coherence return null
+        return nullptr;
+    }
     return item;
 }
 
 QUrl PagePool::resolvedUrl(const QString &stringUrl) const
 {
-    Q_ASSERT(qmlEngine(this));
-    QQmlContext *ctx = QQmlEngine::contextForObject(this);
+    const auto ctx = qmlContext(this);
     Q_ASSERT(ctx);
 
     QUrl actualUrl(stringUrl);
@@ -276,17 +279,17 @@ void PagePool::deletePage(const QVariant &page)
 
 void PagePool::clear()
 {
-    for (auto *c : std::as_const(m_componentForUrl)) {
-        c->deleteLater();
+    for (const auto &component : std::as_const(m_componentForUrl)) {
+        component->deleteLater();
     }
     m_componentForUrl.clear();
 
-    for (auto *i : std::as_const(m_itemForUrl)) {
+    for (const auto &item : std::as_const(m_itemForUrl)) {
         // items that had been deparented are safe to delete
-        if (!i->parentItem()) {
-            i->deleteLater();
+        if (!item->parentItem()) {
+            item->deleteLater();
         }
-        QQmlEngine::setObjectOwnership(i, QQmlEngine::JavaScriptOwnership);
+        QQmlEngine::setObjectOwnership(item, QQmlEngine::JavaScriptOwnership);
     }
     m_itemForUrl.clear();
     m_urlForItem.clear();
